package com.gihon.sso.auth;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingClass;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import com.gihon.common.auth.AuthConstans;
import com.gihon.common.auth.AuthUser;
import com.gihon.common.auth.AuthUtils;
import com.gihon.common.enums.ModuleType;
import com.gihon.common.exception.LoginException;
import com.gihon.common.properties.GihonCommonProperties;
import com.gihon.common.properties.RedisConstants;
import com.gihon.common.web.response.ResponseStatus;
import com.gihon.sso.service.TokenService;

import lombok.extern.slf4j.Slf4j;

/**
 * @description 认证拦截器
 * 非Security
 * @author baihe
 *
 */
@ConditionalOnMissingClass("org.springframework.security.core.userdetails.UserDetails")
@Order(2)
@Slf4j
@Component
public class AuthenticationInterceptor implements HandlerInterceptor {

    @Resource
    private TokenService tokenService;
    @Resource
    private AuthService authService;
    @Resource
    private GihonCommonProperties gihonCommonProperties;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (request.getMethod().equals(HttpMethod.OPTIONS.name())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }

        String token = (String) request.getAttribute(AuthUtils.REQ_TOKEN);
        if (StringUtils.isBlank(token)) {
            // TODO 未登录
            throw new LoginException(ResponseStatus.UNAUTHORIATION);
        }
        AuthUser user = AuthUtils.getUser();
        if (user == null) {
            // TODO token 过期TOKEN_EXPIRED
            throw new LoginException(ResponseStatus.TOKEN_EXPIRED);

        }
        // 认证token是否合法
        // 超级管理员只要是web平台不校验权限
        if (AuthConstans.ADMIN.equals(user.getUsername()) && gihonCommonProperties.getModuleType() == ModuleType.WEB.getCode()) {
            return true;
        } else if (AuthConstans.ADMIN.equals(user.getUsername())) {
            return false;
        }
        // 动态鉴权
        boolean checkPermission = authService.checkPermission(request.getMethod() + RedisConstants.SEP + getUri(request), gihonCommonProperties.getModuleType(),
                token, user);
        if (!checkPermission) {
            throw new LoginException(ResponseStatus.FORBIDDEN);
        }
        return true;
    }

    //获取资源的路径,去掉ContextPath
    private String getUri(HttpServletRequest request) {
        String uri = request.getContextPath();
        String url = request.getRequestURI();
//    	String url3 = request.getServletPath();
        if (uri.equals("/")) {
            return url;
        } else {
            return url.replace(uri, "");
        }
    }

}
