package com.gihon.sso.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.common.entity.GihonMenu;
import com.gihon.common.exception.BusinessException;
import com.gihon.common.mapper.GihonMenuMapper;
import com.gihon.sso.service.GihonMenuService;

/**
 * 直接继承没有多实现处理
 * 
 * @author baihe
 *
 */
@Service
public class GihonMenuServiceImpl extends ServiceImpl<GihonMenuMapper, GihonMenu> implements GihonMenuService{

    /**
     * 获取模块下全部菜单
     */
    public List<GihonMenu> getAllMenu(Long moduleId) {
        List<GihonMenu> rootList = new ArrayList<>();
        getMenuList(moduleId, -1L, rootList);
        return rootList;
    }

    /**
     * 获取模块下子菜单
     */
    public List<GihonMenu> getSubMenu(Long moduleId, Long menuId) {
        return this.lambdaQuery().eq(GihonMenu::getModuleId, moduleId).eq(GihonMenu::getParentId, menuId).orderByAsc(GihonMenu::getOrderNum).list();
    }

    /**
     * 递归
     * 
     * @param MenuId
     * @param menuId
     * @param result
     */
    private void getMenuList(Long moduleId, Long menuId, List<GihonMenu> result) {
        List<GihonMenu> sublist = this.getSubMenu(moduleId, menuId);
        if (sublist.isEmpty()) {
            return;
        }
        result.addAll(sublist);
        for (GihonMenu menu : sublist) {
            getMenuList(moduleId, menu.getId(), result);
        }
    }

    /**
     * 新增菜单
     */
    public boolean saveMenu(GihonMenu menu) {
        int count = this.lambdaQuery().eq(GihonMenu::getModuleId, menu.getModuleId()).and(p -> {
            p.eq(GihonMenu::getMenuName, menu.getMenuName()).or().eq(GihonMenu::getMenuCode, menu.getMenuCode());
        }).count();
        if (count > 0) {
            throw new BusinessException("菜单编码或者名称重复");
        }
        return this.save(menu);

    }

    /**
     * 编辑菜单
     */
    public boolean updateMenu(GihonMenu menu) {
        GihonMenu menuOld = this.getById(menu.getId());
        if (menuOld == null) {
            throw new BusinessException("菜单不存在");
        }
        int count = this.lambdaQuery().eq(GihonMenu::getModuleId, menu.getModuleId()).ne(GihonMenu::getId, menu.getId()).and(p -> {
            p.eq(GihonMenu::getMenuName, menu.getMenuName()).or().eq(GihonMenu::getMenuCode, menu.getMenuCode());
        }).count();
        if (count > 0) {
            throw new BusinessException("菜单编码或者名称重复");
        }
        GihonMenu menuUpdate = new GihonMenu();
        menuUpdate.setId(menu.getId());
        menuUpdate.setMenuCode(menu.getMenuCode());
        menuUpdate.setMenuName(menu.getMenuName());
        menuUpdate.setOrderNum(menu.getOrderNum());
        menuUpdate.setPath(menu.getPath());

        return this.updateById(menuUpdate);

    }
}
